package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special;

import com.mojang.datafixers.kinds.Applicative;
import net.minecraft.core.registries.BuiltInRegistries;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import net.minecraft.ChatFormatting;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import net.minecraft.network.chat.Component;
import java.util.Iterator;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import java.util.UUID;
import java.util.function.BiConsumer;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.List;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;

public class AllStatsBonus extends GemBonus
{
    public static Codec<AllStatsBonus> CODEC;
    protected final AttributeModifier.Operation operation;
    protected final Map<LootRarity, StepFunction> values;
    protected final List<Attribute> attributes;
    
    public AllStatsBonus(final GemClass gemClass, final AttributeModifier.Operation op, final Map<LootRarity, StepFunction> values, final List<Attribute> attributes) {
        super(Apotheosis.loc("all_stats"), gemClass);
        this.operation = op;
        this.values = values;
        this.attributes = attributes;
    }
    
    public void addModifiers(final ItemStack gem, final LootRarity rarity, final BiConsumer<Attribute, AttributeModifier> map) {
        final UUID id = GemItem.getUUIDs(gem).get(0);
        for (final Attribute attr : this.attributes) {
            final AttributeModifier modif = new AttributeModifier(id, "apoth.gem_modifier.all_stats_buff", (double)this.values.get(rarity).min(), this.operation);
            map.accept(attr, modif);
        }
    }
    
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final StepFunction value = this.values.get(rarity);
        return (Component)Component.m_237110_("bonus." + this.getId() + ".desc", new Object[] { Affix.fmt(value.get(0.0f) * 100.0f) }).m_130940_(ChatFormatting.YELLOW);
    }
    
    public AllStatsBonus validate() {
        Preconditions.checkNotNull(this.operation, (Object)"Invalid AllStatsBonus with null operation");
        Preconditions.checkNotNull(this.values, (Object)"Invalid AllStatsBonus with null values");
        return this;
    }
    
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    public int getNumberOfUUIDs() {
        return 1;
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return AllStatsBonus.CODEC;
    }
    
    static {
        AllStatsBonus.CODEC = (Codec<AllStatsBonus>)RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)PlaceboCodecs.enumCodec((Class)AttributeModifier.Operation.class).fieldOf("operation").forGetter(a -> a.operation), (App)AllStatsBonus.VALUES_CODEC.fieldOf("values").forGetter(a -> a.values), (App)BuiltInRegistries.f_256951_.m_194605_().listOf().fieldOf("attributes").forGetter(a -> a.attributes)).apply((Applicative)inst, AllStatsBonus::new));
    }
}
